"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.scaleBetween = exports.createQuery = exports.createQueryHistoryAutocompleter = void 0;
const compass_components_1 = require("@mongodb-js/compass-components");
const mongodb_query_parser_1 = require("mongodb-query-parser");
const compass_components_2 = require("@mongodb-js/compass-components");
const createQueryHistoryAutocompleter = (savedQueries, onApply) => {
    return function queryCompletions(context) {
        if (savedQueries.length === 0) {
            return null;
        }
        const maxTime = savedQueries[savedQueries.length - 1].lastExecuted.getTime();
        const minTime = savedQueries[0].lastExecuted.getTime();
        const options = savedQueries.map((query) => ({
            label: createQuery(query),
            type: 'text',
            detail: (0, compass_components_1.formatDate)(query.lastExecuted.getTime()),
            info: () => createInfo(query).dom,
            apply: () => {
                onApply(query.queryProperties);
            },
            boost: scaleBetween(query.lastExecuted.getTime(), -99, 99, minTime, maxTime),
        }));
        return {
            from: context.pos,
            options: options,
        };
    };
};
exports.createQueryHistoryAutocompleter = createQueryHistoryAutocompleter;
const queryLabelStyles = (0, compass_components_2.css)({
    textTransform: 'capitalize',
    fontWeight: 'bold',
    margin: `${compass_components_1.spacing[2]}px 0`,
});
const queryCodeStyles = (0, compass_components_2.css)({
    maxHeight: '30vh',
});
const completionInfoStyles = (0, compass_components_2.css)({
    overflow: 'auto',
});
function createQuery(query) {
    let res = '';
    Object.entries(query.queryProperties).forEach(([key, value]) => {
        const formattedQuery = (0, mongodb_query_parser_1.toJSString)(value);
        const noFilterKey = key === 'filter' ? '' : `${key}: `;
        res += formattedQuery ? `, ${noFilterKey}${formattedQuery}` : '';
    });
    const len = res.length;
    return len <= 100 ? res.slice(2, res.length) : res.slice(2, 100);
}
exports.createQuery = createQuery;
function createInfo(query) {
    const container = document.createElement('div');
    container.className = completionInfoStyles;
    Object.entries(query.queryProperties).forEach(([key, value]) => {
        const formattedQuery = (0, mongodb_query_parser_1.toJSString)(value);
        const codeDiv = document.createElement('div');
        const label = document.createElement('label');
        label.className = queryLabelStyles;
        label.textContent = key;
        const code = document.createElement('pre');
        code.className = queryCodeStyles;
        if (formattedQuery)
            code.textContent = formattedQuery;
        codeDiv.append(label);
        codeDiv.appendChild(code);
        container.appendChild(codeDiv);
    });
    return {
        dom: container,
        destroy: () => {
            while (container.firstChild) {
                container.removeChild(container.firstChild);
            }
        },
    };
}
function scaleBetween(unscaledNum, newScaleMin, newScaleMax, originalScaleMin, originalScaleMax) {
    if (originalScaleMax === originalScaleMin)
        return newScaleMin + (newScaleMax - newScaleMin) / 2;
    return (((newScaleMax - newScaleMin) * (unscaledNum - originalScaleMin)) /
        (originalScaleMax - originalScaleMin) +
        newScaleMin);
}
exports.scaleBetween = scaleBetween;
//# sourceMappingURL=query-history-autocompleter.js.map